<?php if ( ! defined('APP_ABSPATH')) exit('No direct script access allowed');
class Controller {
	var $load;
	var $model;
	var $input;
	var $helpers;
	var $router;

	/*
	*	The Constructor class
	*	Creates instances of the classes that will be needed here
	*	Using the Router class it then calls the function that corresponds to the url
	*/
	function Controller($legacy_call = false){
		$this->view = new View();
		$this->model = new Model();
		$this->input = new Input();
		$this->helpers = new Helpers();
	}

	/*
	*	This is the default method. It displays the home page of the admin dashboard
	*/
	function index(){
		$this->login();
	}
	
	function admin_home(){
		$this->login_check();
		$this->admin_list_banners();
	}
	
	function admin_banners(){
		$this->login_check();
		$this->admin_list_banners();
	}
	
	function admin_create_banner_slot(){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		if(isset($_POST['create'])){
			if($_POST['banner_type'] == '')
				$data['error'] .= ' The Banner Type is required.';
			if($_POST['name'] == '')
				$data['error'] .= ' The Name is required.';
			if($_POST['description'] == '')
				$data['error'] .= ' The Description is required.';
			if($_POST['width'] == '')
				$data['error'] .= ' The Width is required.';
			if($_POST['height'] == '')
				$data['error'] .= ' The Height is required.';
			if($_POST['plans_no'] > 0){
				for($i = 1; $i <= $_POST['plans_no']; $i++){
					if(($_POST['price_' . $i] == '') OR ($_POST['price_' . $i] == 'Price'))
						$data['error'] .= ' The price for Price Plan ' . $i . ' is required.';
					if(($_POST['quantity_' . $i] == '') OR ($_POST['quantity_' . $i] == 'Quantity'))
						$data['error'] .= ' The quantity for Price Plan ' . $i . ' is required.';
				}
			}
				
			if($data['error'] == ''){
				$save = array('name'=>$this->input->post('name'),
								'banner_type'=>$this->input->post('banner_type'),
								'description'=>$this->input->post('description'),
								'width'=>$this->helpers->convert_to_number($this->input->post('width')),
								'height'=>$this->helpers->convert_to_number($this->input->post('height')),
								'numberofads'=>$this->input->post('no_of_ads'));
				
				$new_slot = $this->model->create_slot($save);
				if($new_slot){
					if($_POST['plans_no'] > 0){
						for($i = 1; $i <= $_POST['plans_no']; $i++){
							$price = array('space_id'=>$new_slot,
											'unit'=>$this->input->post('unit_' . $i),
											'price'=>$this->input->post('price_' . $i),
											'quantity'=>$this->input->post('quantity_' . $i));
							$this->model->create_price($price);
						}
					}
					$data['success'] .= ' The new advertisement slot has been created.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$this->view->render('admin/banners/create_advert_slot.php', $data);
	}
	
	function admin_list_banner_slots($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check();
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$data['banner_slots'] = $this->model->get_paged_banner_slots($offset, $rows);
		$data['no_of_slots'] = $this->model->count_banner_slots();
		$data['number_of_pages'] = ceil( $data['no_of_slots'] / $rows );
		$data['page'] = $page;
		$this->view->render('admin/banners/list_slots.php', $data);
		
	}
	
	function admin_edit_banner_slot($id){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		$data['new_price_plans'] = 0;
		
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_banner_slot', array($id)));
			exit();
		}
		
		
		if(isset($_POST['edit'])){
			if($_POST['banner_type'] == '')
				$data['error'] .= ' The Banner Type is required.';
			if($_POST['name'] == '')
				$data['error'] .= ' The Name is required.';
			if($_POST['description'] == '')
				$data['error'] .= ' The Description is required.';
			if($_POST['width'] == '')
				$data['error'] .= ' The Width is required.';
			if($_POST['height'] == '')
				$data['error'] .= ' The Height is required.';
			$price_plans = $this->model->get_slot_prices($id);
			if($_POST['plans_no'] > count($price_plans)){
				for($i = (1 +  count($price_plans)); $i <= $_POST['plans_no']; $i++){
					if(($_POST['price_' . $i] == '') OR ($_POST['price_' . $i] == 'Price'))
						$data['error'] .= ' The price for Price Plan ' . $i . ' is required.';
					if(($_POST['quantity_' . $i] == '') OR ($_POST['quantity_' . $i] == 'Quantity'))
						$data['error'] .= ' The quantity for Price Plan ' . $i . ' is required.';
				}
			}
				
			if($data['error'] == ''){
				$save = array('name'=>$this->input->post('name'),
								'banner_type'=>$this->input->post('banner_type'),
								'description'=>$this->input->post('description'),
								'width'=>$this->helpers->convert_to_number($this->input->post('width')),
								'height'=>$this->helpers->convert_to_number($this->input->post('height')),
								'numberofads'=>$this->input->post('no_of_ads'));
				
				$update_slot = $this->model->update_slot($id, $save);
				
				if($_POST['plans_no'] > count($price_plans)){
					for($i = (1 +  count($price_plans)); $i <= $_POST['plans_no']; $i++){
						$price = array('space_id'=>$id,
											'unit'=>$this->input->post('unit_' . $i),
											'price'=>$this->input->post('price_' . $i),
											'quantity'=>$this->input->post('quantity_' . $i));
						$this->model->create_price($price);
					}
				}
				$deleted_price_plans = 0;
				foreach ($_POST as $k => $v){
					if (substr($k,0,13) == "delete_price_"){
						$price_id = intval($v);
						$this->model->delete_price($price_id);
						$deleted_price_plans = $deleted_price_plans + 1;
					}
				}
				
				$data['success'] .= ' The banner slot has been updated.';
			}
		}
		$data['slot'] = $this->model->get_slot($id);
		$data['prices'] = $this->model->get_slot_prices($id);
		$data['new_price_plans'] = $_POST['plans_no'] - $deleted_price_plans;
		
		$this->view->render('admin/banners/edit_advert_slot.php', $data);
	}
	
	function refresh_slot($id){
		$slot = $this->model->get_slot($id);
		
		$output = array();
		
		$output['status'] = 1;
		$space_height = intval($slot['height']);
		$padding = ($space_height/2) - 12;
		$height = $space_height - $padding;

		
		$output['markup'] = '&lt;!-- START: Simple Adverts : '. $slot['name'] .' --&gt;
&lt;div rel="simple_advert" style="display:block;padding:0px;width:'. $slot['width'] .'px; height:'. $slot['height'] .'px;" id="sa_'. $slot['id'] .'_'. time().'"&gt;
&lt;div class="simple_advert" style="width:100%; height:100%;"&gt;
&lt;a title="Advertise Here" target="_blank" style="padding-top: '.$padding.'px;width:'.$slot['width'].'px;height:'.$height.'px;" href="' . $this->helpers->create_url('buy_slot', array($slot['id'])) . '"&gt;Advertise Here&lt;/a&gt;
&lt;/div&gt;
&lt;/div&gt;
&lt;!-- END: Simple Adverts : '. $slot['name'] .' --&gt;';
		
		echo json_encode($output);
		exit();
	}
	
	function admin_delete_banner_slot($id){
		$this->login_check();
		$data = array();
		$data['slot_id'] = $id;
		$slot = $this->model->get_slot($id);
		if(isset($_POST['delete_slot'])){
			$this->model->delete_slot($id);
			$this->model->delete_slot_prices($id);
			$data['success'] = 'The banner slot has been delete.';
		}
		$this->view->render('admin/banners/delete_advert_slot.php', $data);
	}
	
	function admin_create_user(){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		if(isset($_POST['create'])){
			if($_POST['name'] == '')
				$data['error'] .= ' The Name is required.';
			if($_POST['email'] == '')
				$data['error'] .= ' The Email is required.';
			if($this->model->email_exists($_POST['email']) == false)
				$data['error'] .= ' The Email is already in use. Please select another one.';
			if($_POST['password'] == '')
				$data['error'] .= ' The Password is required.';
			if($_POST['password'] != $_POST['cpassword'])
				$data['error'] .= ' The Password does not match the confirmation password.';
			if($_POST['role'] == '')
				$data['error'] .= ' The Role is required.';
				
			if($data['error'] == ''){
				$save = array('name'=>$this->input->post('name'),
								'email'=>$this->input->post('email'),
								'password'=>md5($this->input->post('password')),
								'role'=>$this->input->post('role'));
				
				if($_POST['company'] != '')
					$save['company'] = $this->input->post('company');
				$new_user = $this->model->create_user($save);
				if($new_user){
					if($_POST['send_email']){
						$email_data = array();
						$email_data['name'] = $this->input->post('name');
						$email_data['email'] = $this->input->post('email');
						$email_data['password'] = $this->input->post('password');
						$text = $this->view->get('email_templates/new_user_text.php', $email_data);
						$html = $this->view->get('email_templates/new_user_html.php', $email_data);
						
						$this->helpers->html_email(APP_WEBMASTER_EMAIL, $this->input->post('email'), 'User Account Details[' . $_SERVER['SERVER_NAME'] . ']', $text, $html);
					}
					
					$data['success'] .= ' The user has been created.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$this->view->render('admin/users/create_user.php', $data);
	}
	
	function admin_users(){
		$this->login_check();
		$this->admin_list_users();
	}
	
	function admin_list_users($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check();
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$data['users'] = $this->model->get_paged_users($offset, $rows);
		$data['no_of_users'] = $this->model->count_users();
		$data['number_of_pages'] = ceil( $data['no_of_users'] / $rows );
		$data['page'] = $page;
		$this->view->render('admin/users/list_users.php', $data);
		
	}
	
	function admin_edit_user($id){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_user', array($id)));
			exit();
		}
		
		if(isset($_POST['edit'])){
			if($_POST['name'] == '')
				$data['error'] .= ' The Name is required.';
			if($_POST['email'] == '')
				$data['error'] .= ' The Email is required.';
			if(($_POST['email'] != $_POST['old_email']) AND ($this->model->email_exists($_POST['email']) == false))
				$data['error'] .= ' The Email is already in use. Please select another one.';
			if($_POST['role'] == '')
				$data['error'] .= ' The Role is required.';
				
			if($data['error'] == ''){
				$save = array('name'=>$this->input->post('name'),
								'company'=>$this->input->post('company'),
								'email'=>$this->input->post('email'),
								'role'=>$this->input->post('role'));
				
				$new_user = $this->model->update_user($id, $save);
				$data['success'] .= ' The user has been updated.';
			}
		}
		$data['user'] = $this->model->get_user($id);
		$this->view->render('admin/users/edit_user.php', $data);
	}
	
	function admin_delete_user($id){
		$this->login_check();
		$data = array();
		$data['user_id'] = $id;
		
		if(isset($_POST['delete_user'])){
			$this->model->delete_user($id);
			$data['success'] = 'The user has been delete.';
		}
		$this->view->render('admin/users/delete_user.php', $data);
	}
	
	function admin_create_image_banner(){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		if(isset($_POST['create'])){
			if($_POST['slot'] == '')
				$data['error'] .= ' The Banner slot is required.';
			if($_POST['title'] == '')
				$data['error'] .= ' The Headline of the banner is required.';
			if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
				$data['error'] .= ' The Destination URL is required.';
			if($_POST['startdate'] == '')
				$data['error'] .= ' The Start Date is required.';
			
			if($_FILES["image"]["error"] > 0){
				$data['error'] .= $_FILES["image"]["error"];
			} else {
				if(!$this->helpers->valid_file_extension($_FILES["image"]["name"]))
					$data['error'] .= ' The uploaded file is of an invalid type (it should be an image or swf).';
				if($_POST['slot'] != ''){
					if($this->helpers->get_extension($_FILES["image"]["name"]) != 'swf'){
						list($image_width, $image_height) = getimagesize($_FILES["image"]["tmp_name"]);
						$selected_slot = $this->model->get_slot($_POST['slot']);
						if($image_width != $selected_slot['width'])
							$data['error'] .= ' The uploaded file should have a width of ' . $selected_slot['width'] . 'px.';
						if($image_height != $selected_slot['height'])
							$data['error'] .= ' The uploaded file should have a height of ' . $selected_slot['height'] . 'px.';
					}
				}
			}
				
			if($data['error'] == ''){
				$filename = $this->helpers->set_filename("images/adverts/", $_FILES["image"]["name"]);
				$save = array('title'=>$this->input->post('title'),
								'destination_url'=>$this->input->post('destination_url'),
								'group_or_slot_id'=>$this->input->post('slot'),
								'startdate'=>$this->helpers->convert_date($this->input->post('startdate')),
								'image'=>$filename,
								'advert_type'=>'banner');
				if($_POST['advertiser'] != '')
					$save['advertiser_id'] = $this->input->post('advertiser');
				if($_POST['enddate'] != '')
					$save['enddate'] = $this->helpers->convert_date($this->input->post('enddate'));	
				if($_POST['impressions'] != '')
					$save['maximpressions'] = $this->helpers->convert_to_number($this->input->post('impressions'));
				if($_POST['clicks'] != '')
					$save['maxclicks'] = $this->helpers->convert_to_number($this->input->post('clicks'));
				
				$new_advert = $this->model->create_advert($save);
				if($new_advert){
					move_uploaded_file($_FILES["image"]["tmp_name"], "images/adverts/" . $filename);
					$data['success'] .= ' The new advertisement has been created.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$data['slots'] = $this->model->get_all_banner_slots();
		if($data['slots']['banner_type'] != 'multi_text'){
			$data['advertisers'] = $this->model->get_advertisers();
			$this->view->render('admin/banners/create_image_banner.php', $data);
		} else {
			$this->admin_create_text_banner();
		}
	}
	
	function admin_create_text_banner(){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		if(isset($_POST['create'])){
			if($_POST['slot'] == '')
				$data['error'] .= ' The Banner slot is required.';
			if($_POST['title'] == '')
				$data['error'] .= ' The Headline of the banner is required.';
			if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
				$data['error'] .= ' The Destination URL is required.';
			if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
				$data['error'] .= ' The Display URL is required.';
			if(($_POST['details1'] == '') AND ($_POST['details2'] == ''))
				$data['error'] .= ' At least one line of description is required.';
			if($_POST['startdate'] == '')
				$data['error'] .= ' The Start Date is required.';
			
			
				
			if($data['error'] == ''){
				$save = array('title'=>$this->input->post('title'),
								'destination_url'=>$this->input->post('destination_url'),
								'display_url'=>$this->input->post('display_url'),
								'group_or_slot_id'=>$this->input->post('slot'),
								'startdate'=>$this->helpers->convert_date($this->input->post('startdate')),
								'advert_type'=>'banner');
				if($_POST['advertiser'] != '')
					$save['advertiser_id'] = $this->input->post('advertiser');
				if($_POST['enddate'] != '')
					$save['enddate'] = $this->helpers->convert_date($this->input->post('enddate'));	
				if($_POST['impressions'] != '')
					$save['maximpressions'] = $this->helpers->convert_to_number($this->input->post('impressions'));
				if($_POST['clicks'] != '')
					$save['maxclicks'] = $this->helpers->convert_to_number($this->input->post('clicks'));
				if($_POST['details1'] != '')
					$save['details1'] = $this->input->post('details1');
				if($_POST['details2'] != '')
					$save['details2'] = $this->input->post('details2');
				
				$new_advert = $this->model->create_advert($save);
				if($new_advert){
					$data['success'] .= ' The new advertisement has been created.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$data['slots'] = $this->model->get_all_banner_slots();
		$data['advertisers'] = $this->model->get_advertisers();
		$this->view->render('admin/banners/create_text_banner.php', $data);
	}
	
	function admin_list_banners($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check();
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$raw_banners = $this->model->get_paged_banner_adverts($offset, $rows);
		if($raw_banners){
			for($i = 0; $i < count($raw_banners); $i++){
				$raw_banners[$i]['is_live'] = $this->is_live($raw_banners[$i]);
			}
		}
		$data['banners'] = $raw_banners;
		
		
		$data['no_of_banners'] = $this->model->count_banner_adverts();
		$data['number_of_pages'] = ceil( $data['no_of_banners'] / $rows );
		$data['page'] = $page;
		$this->view->render('admin/banners/list_banners.php', $data);
		
	}
	
	function is_live($banner){
		$islive = true;
		
		if($banner['stopped'] == 'y'){
			$islive = false;
		}
		if($banner['enddate']){
			if($banner['enddate'] < time())
				$islive = false;
		}
		if($banner['maximpressions']){
			if($banner['impressions'] >= $banner['maximpressions'])
				$islive = false;
		}
		if($banner['maxclicks']){
			if($banner['clicks'] >= $banner['maxclicks'])
				$islive = false;
		}

		return $islive;
	}
	
	function admin_edit_image_banner($id){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_banner', array($id)));
			exit();
		}
		
		if(isset($_POST['edit'])){
			if($_POST['slot'] == '')
				$data['error'] .= ' The Banner slot is required.';
			if($_POST['title'] == '')
				$data['error'] .= ' The Headline of the banner is required.';
			if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
				$data['error'] .= ' The Destination URL is required.';
			if($_POST['startdate'] == '')
				$data['error'] .= ' The Start Date is required.';
			
			if($_FILES["image"]["name"]){
				
				if($_FILES["image"]["error"] > 0){
					$data['error'] .= $_FILES["image"]["error"];
				} else {
					if(!$this->helpers->valid_file_extension($_FILES["image"]["name"]))
						$data['error'] .= ' The uploaded file is of an invalid type (it should be an image or swf).';
					if($_POST['slot'] != ''){
						if($this->helpers->get_extension($_FILES["image"]["name"]) != 'swf'){
							list($image_width, $image_height) = getimagesize($_FILES["image"]["tmp_name"]);
							$selected_slot = $this->model->get_slot($_POST['slot']);
							if($image_width != $selected_slot['width'])
								$data['error'] .= ' The uploaded file should have a width of ' . $selected_slot['width'] . 'px.';
							if($image_height != $selected_slot['height'])
								$data['error'] .= ' The uploaded file should have a height of ' . $selected_slot['height'] . 'px.';
						}
					}
				}
			}
				
			if($data['error'] == ''){
				$save = array('title'=>$this->input->post('title'),
								'destination_url'=>$this->input->post('destination_url'),
								'group_or_slot_id'=>$this->input->post('slot'),
								'startdate'=>$this->helpers->convert_date($this->input->post('startdate')),
								'advertiser_id'=>$this->input->post('advertiser'),
								'enddate'=>$this->helpers->convert_date($this->input->post('enddate')),
								'maximpressions'=>$this->helpers->convert_to_number($this->input->post('impressions')),
								'maxclicks'=>$this->helpers->convert_to_number($this->input->post('clicks')));
				
				if(isset($_POST['stop'])){
					$save['stopped '] = 'y';
				} else {
					$save['stopped '] = 'n';
				}
				
				if($_FILES["image"]["name"]){
					
					$filename = $this->helpers->set_filename("images/adverts/", $_FILES["image"]["name"]);
					$save['image'] = $filename;
				}
				$update_advert = $this->model->update_advert($id, $save);
				if($update_advert){
					if($_FILES["image"]["name"]){
						move_uploaded_file($_FILES["image"]["tmp_name"], "images/adverts/" . $filename);
						$file = "images/adverts/" . $this->input->post('old_image');
						$exists = file_exists($file);
						if($exists){
							unlink($file);
						}
					}
					
					$data['success'] .= ' The banner advertisement has been updated.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$data['banner'] = $this->model->get_advert($id);
		$data['banner']['is_live'] = $this->is_live($data['banner']);
		$data['slots'] = $this->model->get_all_banner_slots();
		$data['advertisers'] = $this->model->get_advertisers();
		$this->view->render('admin/banners/edit_image_banner.php', $data);
	}
	
	function admin_edit_text_banner($id){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_banner', array($id)));
			exit();
		}
		
		if(isset($_POST['edit'])){
			if($_POST['slot'] == '')
				$data['error'] .= ' The Banner slot is required.';
			if($_POST['title'] == '')
				$data['error'] .= ' The Headline of the banner is required.';
			if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
				$data['error'] .= ' The Destination URL is required.';
			if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
				$data['error'] .= ' The Display URL is required.';
			if(($_POST['details1'] == '') AND ($_POST['details2'] == ''))
				$data['error'] .= ' At least one line of description is required.';
			if($_POST['startdate'] == '')
				$data['error'] .= ' The Start Date is required.';
			
			
				
			if($data['error'] == ''){
				$save = array('title'=>$this->input->post('title'),
								'destination_url'=>$this->input->post('destination_url'),
								'display_url'=>$this->input->post('display_url'),
								'group_or_slot_id'=>$this->input->post('slot'),
								'startdate'=>$this->helpers->convert_date($this->input->post('startdate')),
								'advert_type'=>'banner');
				$save['advertiser_id'] = $this->input->post('advertiser');
				$save['enddate'] = $this->helpers->convert_date($this->input->post('enddate'));	
				$save['maximpressions'] = $this->helpers->convert_to_number($this->input->post('impressions'));
				$save['maxclicks'] = $this->helpers->convert_to_number($this->input->post('clicks'));
				$save['details1'] = $this->input->post('details1');
				$save['details2'] = $this->input->post('details2');
				
				if(isset($_POST['stop'])){
					$save['stopped '] = 'y';
				} else {
					$save['stopped '] = 'n';
				}
				
				$update_advert = $this->model->update_advert($id, $save);
				if($update_advert){
					$data['success'] .= ' The banner advertisement has been updated.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$data['banner'] = $this->model->get_advert($id);
		$data['banner']['is_live'] = $this->is_live($data['banner']);
		$data['slots'] = $this->model->get_all_banner_slots();
		$data['advertisers'] = $this->model->get_advertisers();
		$this->view->render('admin/banners/edit_text_banner.php', $data);
	}
	
	function admin_delete_banner($id){
		$this->login_check();
		$data = array();
		$data['banner_id'] = $id;
		$advert = $this->model->get_advert($id);
		if(isset($_POST['delete_banner'])){
			$this->model->delete_advert($id);
			if($advert['image']){
				$file = "images/adverts/" . $advert['image'];
				$exists = file_exists($file);
				if($exists){
					unlink($file);
				}
			}
			$data['success'] = 'The banner has been delete.';
		}
		$this->view->render('admin/banners/delete_banner.php', $data);
	}
	
	function admin_textlinks(){
		$this->login_check();
		$this->admin_list_textlinks();
	}
	
	function admin_create_textlink_category(){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		if(isset($_POST['create'])){
			if($_POST['name'] == '')
				$data['error'] .= ' The Name is required.';
			if($_POST['description'] == '')
				$data['error'] .= ' The Description is required.';
			if($_POST['plans_no'] > 0){
				for($i = 1; $i <= $_POST['plans_no']; $i++){
					if(($_POST['price_' . $i] == '') OR ($_POST['price_' . $i] == 'Price'))
						$data['error'] .= ' The price for Price Plan ' . $i . ' is required.';
					if(($_POST['quantity_' . $i] == '') OR ($_POST['quantity_' . $i] == 'Quantity'))
						$data['error'] .= ' The quantity for Price Plan ' . $i . ' is required.';
				}
			}
				
			if($data['error'] == ''){
				$save = array('name'=>$this->input->post('name'),
								'description'=>$this->input->post('description'),
								'numberofads'=>$this->input->post('no_of_ads'));
				
				$new_slot = $this->model->create_category($save);
				if($new_slot){
					if($_POST['plans_no'] > 0){
						for($i = 1; $i <= $_POST['plans_no']; $i++){
							$price = array('space_id'=>$new_slot,
											'unit'=>$this->input->post('unit_' . $i),
											'price'=>$this->input->post('price_' . $i),
											'quantity'=>$this->input->post('quantity_' . $i));
							$this->model->create_price($price);
						}
					}
					$data['success'] .= ' The new textlink category has been created.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$this->view->render('admin/textlinks/create_category.php', $data);
	}
	
	function admin_list_textlink_categories($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check();
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$data['textlink_categories'] = $this->model->get_paged_textlink_categories($offset, $rows);
		$data['no_of_categories'] = $this->model->count_textlink_categories();
		$data['number_of_pages'] = ceil( $data['no_of_categories'] / $rows );
		$data['page'] = $page;
		$this->view->render('admin/textlinks/list_categories.php', $data);
		
	}
	
	function admin_edit_textlink_category($id){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		$data['new_price_plans'] = 0;
		
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_textlink_category', array($id)));
			exit();
		}
		
		
		if(isset($_POST['edit'])){
			if($_POST['name'] == '')
				$data['error'] .= ' The Name is required.';
			if($_POST['description'] == '')
				$data['error'] .= ' The Description is required.';
			$price_plans = $this->model->get_slot_prices($id);
			if($_POST['plans_no'] > count($price_plans)){
				for($i = (1 +  count($price_plans)); $i <= $_POST['plans_no']; $i++){
					if(($_POST['price_' . $i] == '') OR ($_POST['price_' . $i] == 'Price'))
						$data['error'] .= ' The price for Price Plan ' . $i . ' is required.';
					if(($_POST['quantity_' . $i] == '') OR ($_POST['quantity_' . $i] == 'Quantity'))
						$data['error'] .= ' The quantity for Price Plan ' . $i . ' is required.';
				}
			}
				
			if($data['error'] == ''){
				$save = array('name'=>$this->input->post('name'),
								'description'=>$this->input->post('description'),
								'numberofads'=>$this->input->post('no_of_ads'));
				
				$update_slot = $this->model->update_category($id, $save);
				
				if($_POST['plans_no'] > count($price_plans)){
					for($i = (1 +  count($price_plans)); $i <= $_POST['plans_no']; $i++){
						$price = array('space_id'=>$id,
											'unit'=>$this->input->post('unit_' . $i),
											'price'=>$this->input->post('price_' . $i),
											'quantity'=>$this->input->post('quantity_' . $i));
						$this->model->create_price($price);
					}
				}
				$deleted_price_plans = 0;
				foreach ($_POST as $k => $v){
					if (substr($k,0,13) == "delete_price_"){
						$price_id = intval($v);
						$this->model->delete_price($price_id);
						$deleted_price_plans = $deleted_price_plans + 1;
					}
				}
				
				$data['success'] .= ' The textlink category has been updated.';
			}
		}
		$data['slot'] = $this->model->get_category($id);
		$data['prices'] = $this->model->get_slot_prices($id);
		$data['new_price_plans'] = $_POST['plans_no'] - $deleted_price_plans;
		
		$this->view->render('admin/textlinks/edit_category.php', $data);
	}
	
	function admin_delete_textlink_category($id){
		$this->login_check();
		$data = array();
		$data['slot_id'] = $id;
		$slot = $this->model->get_category($id);
		if(isset($_POST['delete_category'])){
			$this->model->delete_slot($id);
			$this->model->delete_slot_prices($id);
			$data['success'] = 'The textlink category has been delete.';
		}
		$this->view->render('admin/textlinks/delete_category.php', $data);
	}
	
	function admin_create_textlink(){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		if(isset($_POST['create'])){
			if($_POST['slot'] == '')
				$data['error'] .= ' The Textlink Category is required.';
			if($_POST['title'] == '')
				$data['error'] .= ' The Headline of the texlink is required.';
			if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
				$data['error'] .= ' The Destination URL is required.';
			if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
				$data['error'] .= ' The Display URL is required.';
			if($_POST['startdate'] == '')
				$data['error'] .= ' The Start Date is required.';
			
			
				
			if($data['error'] == ''){
				$save = array('title'=>$this->input->post('title'),
								'destination_url'=>$this->input->post('destination_url'),
								'display_url'=>$this->input->post('display_url'),
								'group_or_slot_id'=>$this->input->post('slot'),
								'startdate'=>$this->helpers->convert_date($this->input->post('startdate')),
								'advert_type'=>'textlink');
				if($_POST['advertiser'] != '')
					$save['advertiser_id'] = $this->input->post('advertiser');
				if($_POST['enddate'] != '')
					$save['enddate'] = $this->helpers->convert_date($this->input->post('enddate'));	
				if($_POST['impressions'] != '')
					$save['maximpressions'] = $this->helpers->convert_to_number($this->input->post('impressions'));
				if($_POST['clicks'] != '')
					$save['maxclicks'] = $this->helpers->convert_to_number($this->input->post('clicks'));
				
				$new_advert = $this->model->create_advert($save);
				if($new_advert){
					$data['success'] .= ' The new textlink has been created.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$data['slots'] = $this->model->get_all_categories();
		$data['advertisers'] = $this->model->get_advertisers();
		$this->view->render('admin/textlinks/create_textlink.php', $data);
	}
	
	function admin_list_textlinks($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check();
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$raw_banners = $this->model->get_paged_textlink($offset, $rows);
		if($raw_banners){
			for($i = 0; $i < count($raw_banners); $i++){
				$raw_banners[$i]['is_live'] = $this->is_live($raw_banners[$i]);
			}
		}
		$data['banners'] = $raw_banners;
		
		
		$data['no_of_banners'] = $this->model->count_textlinks();
		$data['number_of_pages'] = ceil( $data['no_of_banners'] / $rows );
		$data['page'] = $page;
		$this->view->render('admin/textlinks/list_textlinks.php', $data);
		
	}
	
	function admin_edit_textlink($id){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_textlink', array($id)));
			exit();
		}
		
		if(isset($_POST['edit'])){
			if($_POST['slot'] == '')
				$data['error'] .= ' The Banner slot is required.';
			if($_POST['title'] == '')
				$data['error'] .= ' The Headline of the banner is required.';
			if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
				$data['error'] .= ' The Destination URL is required.';
			if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
				$data['error'] .= ' The Display URL is required.';
			if($_POST['startdate'] == '')
				$data['error'] .= ' The Start Date is required.';
			
			
				
			if($data['error'] == ''){
				$save = array('title'=>$this->input->post('title'),
								'destination_url'=>$this->input->post('destination_url'),
								'display_url'=>$this->input->post('display_url'),
								'group_or_slot_id'=>$this->input->post('slot'),
								'startdate'=>$this->helpers->convert_date($this->input->post('startdate')),
								'advert_type'=>'textlink');
				$save['advertiser_id'] = $this->input->post('advertiser');
				$save['enddate'] = $this->helpers->convert_date($this->input->post('enddate'));	
				$save['maximpressions'] = $this->helpers->convert_to_number($this->input->post('impressions'));
				$save['maxclicks'] = $this->helpers->convert_to_number($this->input->post('clicks'));
				
				if(isset($_POST['stop'])){
					$save['stopped '] = 'y';
				} else {
					$save['stopped '] = 'n';
				}
				
				$update_advert = $this->model->update_advert($id, $save);
				if($update_advert){
					$data['success'] .= ' The textlink has been updated.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}
		$data['banner'] = $this->model->get_advert($id);
		$data['banner']['is_live'] = $this->is_live($data['banner']);
		$data['slots'] = $this->model->get_all_categories();
		$data['advertisers'] = $this->model->get_advertisers();
		$this->view->render('admin/textlinks/edit_textlink.php', $data);
	}
	
	function admin_delete_textlink($id){
		$this->login_check();
		$data = array();
		$data['textlink_id'] = $id;
		
		if(isset($_POST['delete_textlink'])){
			$this->model->delete_advert($id);
			$data['success'] = 'The textlink has been delete.';
		}
		$this->view->render('admin/textlinks/delete_textlink.php', $data);
	}
	
	function admin_pending(){
		$this->login_check();
		$this->admin_list_pending();
	}
	
	function admin_list_pending($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check();
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$data['banners'] = $this->model->get_paged_pending($offset, $rows);
		$data['no_of_banners'] = $this->model->count_pending();
		$data['number_of_pages'] = ceil( $data['no_of_banners'] / $rows );
		$data['page'] = $page;
		$data['pending_type'] = 'pending';
		$data['current_page_func'] = 'admin_list_pending';
		$data['next_page_func'] = 'admin_view_pending';
		$this->view->render('admin/pending/list_pending.php', $data);
		
	}
	
	function admin_view_pending($id){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		$data['success'] = '';
		
		$pending_item = $this->model->get_pending($id);
		
		
		$data['item_price'] = $this->model->get_price($pending_item['price_id']);
		$data['item'] = $pending_item;
		
		if(!$pending_item['advertiser_id']){
			$data['advertiser_details'] = array();
		} else {
			$data['advertiser_details'] = $this->model->get_user($pending_item['advertiser_id']);
		}
		
		
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_pending', array($id)));
			exit();
		}
		
		if(isset($_POST['reject'])){
			if($_POST['message'] == '')
				$data['error'] .= ' The Reason for the rejection is required.';
			
			if($data['error'] == ''){
				$this->model->update_pending($id, array('rejected'=>'y'));
				$email_data = array();
				
				if(!$pending_item['advertiser_id']){
					$email_data['name'] = $pending_item['name'];
					$email_data['email'] = $pending_item['email'];
				} else {
					$email_data['name'] = $data['advertiser_details']['name'];
					$email_data['email'] = $data['advertiser_details']['email'];
				}
				
				$email_data['reason'] = $this->input->post('message');
				
				$subject = '';
				
				if($pending_item['type_of_pending'] == 'purchase'){
					$subject = 'Your advert was not approved[' . $_SERVER['SERVER_NAME'] . ']';
					$email_data['nature'] = 'advetisement space purchase';
				}
				if($pending_item['type_of_pending'] == 'change'){
					$subject = 'Your changes were not approved[' . $_SERVER['SERVER_NAME'] . ']';
					$email_data['nature'] = 'advetisement change';
				}
				if($pending_item['type_of_pending'] == 'extend'){
					$subject = 'Your advert extension was not approved[' . $_SERVER['SERVER_NAME'] . ']';
					$email_data['nature'] = 'advetisement extensions';
				}
				if($subject == ''){
					$subject = 'Your item was not approved[' . $_SERVER['SERVER_NAME'] . ']';
					$email_data['nature'] = 'item';
				}
				
				$text = $this->view->get('email_templates/rejected_pending_text.php', $email_data);
				$html = $this->view->get('email_templates/rejected_pending_html.php', $email_data);
			
				$this->helpers->html_email(APP_WEBMASTER_EMAIL, $email_data['email'], $subject, $text, $html);
			
				$data['success'] = ' The pending item was rejected.';
			}
		}
		
		if(isset($_POST['approve'])){
			$this->approve_pending($id);
			$this->model->delete_pending($id);
			$data['success'] = ' The pending item has been approved.';
		}
		
		
		$this->view->render('admin/pending/view_pending.php', $data);
		
	}
	
	function admin_list_pending_rejected($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check();
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$data['banners'] = $this->model->get_paged_rejected($offset, $rows);
		$data['no_of_banners'] = $this->model->count_rejected();
		$data['number_of_pages'] = ceil( $data['no_of_banners'] / $rows );
		$data['page'] = $page;
		$data['pending_type'] = 'rejected';
		$data['current_page_func'] = 'admin_list_pending_rejected';
		$data['next_page_func'] = 'admin_view_pending_rejected';
		$this->view->render('admin/pending/list_pending.php', $data);
		
	}
	
	function admin_view_pending_rejected($id){
		$this->login_check();
		$data = array();
		$data['error'] = '';
		$data['success'] = '';
		
		$pending_item = $this->model->get_pending($id);
		$data['item_price'] = $this->model->get_price($pending_item['price_id']);
		$data['item'] = $pending_item;
		
		if(!$pending_item['advertiser_id']){
			$data['advertiser_details'] = array();
		} else {
			$data['advertiser_details'] = $this->model->get_user($pending_item['advertiser_id']);
		}
		
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_pending', array($id)));
			exit();
		}
		
		if(isset($_POST['approve'])){
			$this->approve_pending($id);
			$this->model->delete_pending($id);
			$data['success'] = ' The pending item has been approved.';
		}
		
		$this->view->render('admin/pending/view_rejected.php', $data);
	}
	
	function approve_pending($id){
		$pending = $this->model->get_pending($id);
		$this->model->update_pending($id, array('approved'=>'y'));
				
		if(!$pending['advertiser_id']){
			$advertiser_details = array();
		} else {
			$advertiser_details = $this->model->get_user($pending['advertiser_id']);
		}
				
		if($pending['type_of_pending'] == 'purchase'){
					
			$password = '';
			if(!$pending['advertiser_id']){
				$password = $this->generatepassword();
				$user_data = array('name'=>$pending['name'],
								'email'=>$pending['email'],
								'password'=>md5($password),
								'company'=>$pending['company'],
								'role'=>'advertiser');
				$advertiser_id = $this->model->create_user($user_data);
			} else {
				$advertiser_id = $pending['advertiser_id'];
			}
					
			$save = array('title'=>$pending['title'],
									'group_or_slot_id'=>$pending['slot_id'],
									'image'=>$pending['image'],
									'details1'=>$pending['details1'],
									'details2'=>$pending['details2'],
									'destination_url'=>$pending['destination_url'],
									'display_url'=>$pending['display_url'],
									'advertiser_id'=>$advertiser_id,
									'startdate'=>time(),
									'impressions'=>0,
									'clicks'=>0,
									'stopped'=>'n');
					
			$price = $this->model->get_price($pending['price_id']);
			$quantity = $price['quantity'] * $pending['quantity'];
					
			if($price['unit'] == 'days'){
				$save['enddate'] = time() +  ($quantity * 24 * 60 * 60);
			}
			if($price['unit'] == 'impressions'){
				$save['maximpressions '] = $quantity;
			}
			if($price['unit'] == 'clicks'){
				$save['maxclicks '] = $quantity;
			}
					
			if(($pending['advert_type'] == 'image_banner') OR ($pending['advert_type'] == 'text_banner')){
				$save['advert_type'] = 'banner';
			}
			if($pending['advert_type'] == 'textlink'){
				$save['advert_type '] = 'textlink';
			}
			
			$new_advert = $this->model->create_advert($save);
			$email_data = array();
			if(!$pending_item['advertiser_id']){
				$email_data['name'] = $pending['name'];
				$email_data['email'] = $pending['email'];
			} else {
				$email_data['name'] = $advertiser_details['name'];
				$email_data['email'] = $advertiser_details['email'];
			}
			$email_data['password'] = $password;
			$text = $this->view->get('email_templates/new_advert_text.php', $email_data);
			$html = $this->view->get('email_templates/new_advert_html.php', $email_data);
			$subject = 'Purchase of Advertisement Space Approved';
		}
		
		if($pending['type_of_pending'] == 'change'){
			$advert = $this->model->get_advert($pending['advert_id']);
					
			$save = array();
				
			if($pending['title'])
				$save['title'] = $pending['title'];
			if($pending['destination_url'])
				$save['destination_url'] = $pending['destination_url'];
			if($pending['display_url'])
				$save['display_url'] = $pending['display_url'];
			if($pending['details1'])
				$save['details1'] = $pending['details1'];
			if($pending['details2'])
				$save['details2'] = $pending['details2'];
				
			if($pending['image'] != ''){
				$save['image'] = $pending['image'];
				$file = "images/adverts/" . $advert['image'];
				$exists = file_exists($file);
				if($exists){
					unlink($file);
				}
			}
					
			$update_advert = $this->model->update_advert($pending['advert_id'], $save);
					
			$email_data = array();
			$email_data['name'] = $advertiser_details['name'];
			$email_data['email'] = $advertiser_details['email'];
			$text = $this->view->get('email_templates/extend_advert_text.php', $email_data);
			$html = $this->view->get('email_templates/extend_advert_html.php', $email_data);
			$subject = 'Changes to Your Advertisement Have Been Approved';
		}
		$this->helpers->html_email(APP_WEBMASTER_EMAIL, $email_data['email'], $subject, $text, $html);
		return true;
	}
	
	function admin_list_pending_approved($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check();
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$data['banners'] = $this->model->get_paged_approved($offset, $rows);
		$data['no_of_banners'] = $this->model->count_approved();
		$data['number_of_pages'] = ceil( $data['no_of_banners'] / $rows );
		$data['page'] = $page;
		$data['pending_type'] = 'auto approved';
		$data['current_page_func'] = 'admin_list_pending_approved';
		$data['next_page_func'] = 'admin_view_pending_approved';
		$this->view->render('admin/pending/list_pending.php', $data);
		
	}
	
	function admin_view_pending_approved($id){
		$this->login_check();
		$data = array();
		$data['item'] = $this->model->get_pending($id);
		$data['item_price'] = $this->model->get_price($data['item']['price_id']);
		if(isset($_POST['delete'])){
			header('Location: ' . $this->helpers->create_url('admin_delete_pending', array($id)));
			exit();
		}
		$this->view->render('admin/pending/view_approved.php', $data);
	}
	
	function admin_delete_pending($id){
		$this->login_check();
		$data = array();
		$data['pending_id'] = $id;
		
		if(isset($_POST['delete_pending'])){
			$this->model->delete_pending($id);
			$data['success'] = 'The pending item has been delete.';
		}
		$this->view->render('admin/pending/delete.php', $data);
	}
	
	function advertiser_home($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check('advertiser');
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$raw_banners = $this->model->get_advertisers_paged_adverts($offset, $rows, $_SESSION['sadvert_user_id']);
		if($raw_banners){
			for($i = 0; $i < count($raw_banners); $i++){
				$raw_banners[$i]['is_live'] = $this->is_live($raw_banners[$i]);
			}
		}
		$data['banners'] = $raw_banners;
		
		
		$data['no_of_banners'] = $this->model->count_advertisers_adverts($_SESSION['sadvert_user_id']);
		$data['number_of_pages'] = ceil( $data['no_of_banners'] / $rows );
		$data['page'] = $page;
		$this->view->render('advertisers/list_banners.php', $data);
		
	}
	
	function advertiser_stats($id){
		$this->login_check('advertiser');
		$data['advert'] = $this->model->get_advert($id);
		$data['stats'] = $this->model->get_advert_daily_stats($id);
		$this->view->render('advertisers/stats.php', $data);
	}
	
	function advertiser_change($id){
		$this->login_check('advertiser');
		$data = array();
		$data['error'] = '';
		$data['success'] = '';
		
		$data['banner'] = $this->model->get_advert($id);
		$data['banner']['is_live'] = $this->is_live($data['banner']);
		
		if(isset($_POST['edit'])){
			$save = array();
			
			if($_POST['advert_nature'] == 'textlink'){
				if($_POST['title'] == '')
					$data['error'] .= ' The Headline of the banner is required.';
				if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
					$data['error'] .= ' The Destination URL is required.';
				if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
					$data['error'] .= ' The Display URL is required.';
				
				if($data['error'] == ''){
					$save['title'] = $this->input->post('title');
					$save['destination_url'] = $this->input->post('destination_url');
					$save['display_url'] = $this->input->post('display_url');
					$save['advert_type'] = 'textlink';
				}
				
			}
			if($_POST['advert_nature'] == 'text_banner'){
				if($_POST['title'] == '')
					$data['error'] .= ' The Headline of the banner is required.';
				if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
					$data['error'] .= ' The Destination URL is required.';
				if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
					$data['error'] .= ' The Display URL is required.';
				if(($_POST['details1'] == '') AND ($_POST['details2'] == ''))
					$data['error'] .= ' At least one line of description is required.';
				
				if($data['error'] == ''){
					$save['title'] = $this->input->post('title');
					$save['destination_url'] = $this->input->post('destination_url');
					$save['display_url'] = $this->input->post('display_url');
					$save['details1'] = $this->input->post('details1');
					$save['details2'] = $this->input->post('details2');
					$save['advert_type'] = 'text_banner';
				}
				
			}
			if($_POST['advert_nature'] == 'image_banner'){
				if($_POST['title'] == '')
					$data['error'] .= ' The Headline of the banner is required.';
				if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
					$data['error'] .= ' The Destination URL is required.';
				if($_FILES["image"]["name"]){
					if($_FILES["image"]["error"] > 0){
						$data['error'] .= $_FILES["image"]["error"];
					} else {
						if(!$this->helpers->valid_file_extension($_FILES["image"]["name"]))
							$data['error'] .= ' The uploaded file is of an invalid type (it should be an image or swf).';
						list($image_width, $image_height) = getimagesize($_FILES["image"]["tmp_name"]);
						$selected_slot = $this->model->get_slot($data['banner']['group_or_slot_id ']);
						if($image_width != $selected_slot['width'])
							$data['error'] .= ' The uploaded file should have a width of ' . $selected_slot['width'] . 'px.';
						if($image_height != $selected_slot['height'])
							$data['error'] .= ' The uploaded file should have a height of ' . $selected_slot['height'] . 'px.';
					}
				}
				
				if($data['error'] == ''){
					$save['title'] = $this->input->post('title');
					$save['destination_url'] = $this->input->post('destination_url');
					$save['advert_type'] = 'image_banner';
					if($_FILES["image"]["name"]){
						$filename = $this->helpers->set_filename("images/adverts/", $_FILES["image"]["name"]);
						$save['image'] = $filename;
					}
				}
				
			}
			
			if($data['error'] == ''){
				$save['advertiser_id'] = $_SESSION['sadvert_user_id'];
				$save['dateadded'] = time();
				$save['rejected'] = 'n';
				$save['approved'] = 'n';
				$save['confirmed'] = 'y';
				$save['type_of_pending'] = 'change';
				$save['advert_id'] = $id;
				
				$pending = $this->model->create_pending($save);
				if($pending){
					if($_FILES["image"]["name"]){
						move_uploaded_file($_FILES["image"]["tmp_name"], "images/adverts/" . $filename);
					}
					$data['success'] .= ' The changes have been saved. We will approve them and add them to you advertisement.';
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		}

		if($data['banner']['advert_type'] == 'textlink'){
			$this->view->render('advertisers/edit_textlink.php', $data);	
		} else {
			if(!$data['banner']['image']){
				$this->view->render('advertisers/edit_text_banner.php', $data);
			} else {
				$this->view->render('advertisers/edit_image_banner.php', $data);
			}
		}
	}
	
	function advertiser_contact(){
		$this->login_check('advertiser');
		$data = array();
		$data['error'] = '';
		if(isset($_POST['send'])){
			if($_POST['subject'] == '')
				$data['error'] .= ' The Subject is required.';
			if($_POST['body'] == '')
				$data['error'] .= ' The Message is required.';
				
			if($data['error'] == ''){
				$email_data = array();
				
				$user = $this->model->get_user($_SESSION['sadvert_user_id']);
				
				$email_data['name'] = $user['name'];
				$email_data['email'] = $user['email'];
				$email_data['message'] = $this->input->post('body');
				$email_data['subject'] = $this->input->post('subject');
				
				$subject = '[Simple Advert Contact Page]' . strtoupper($this->input->post('subject'));
				$text = $this->view->get('email_templates/contact_text.php', $email_data);
				$html = $this->view->get('email_templates/contact_html.php', $email_data);
				
				$this->helpers->html_email(APP_WEBMASTER_EMAIL, $user['email'], $subject, $text, $html);
				
				$data['success'] .= ' The message has been sent and we will get back to you as soon as possible.';
			}
		}
		$this->view->render('advertisers/contact.php', $data);
	}
	
	function advertiser_change_details(){
		$this->login_check('advertiser');
		$data = array();
		$data['error'] = '';
		
		$id = $_SESSION['sadvert_user_id'];
		
		if(isset($_POST['edit'])){
			if($_POST['name'] == '')
				$data['error'] .= ' Your Name is required.';
			if($_POST['email'] == '')
				$data['error'] .= ' Your Email is required.';
			if(($_POST['email'] != $_POST['old_email']) AND ($this->model->email_exists($_POST['email']) == false))
				$data['error'] .= ' The Email is already in use. Please select another one.';
				
			if($data['error'] == ''){
				$save = array('name'=>$this->input->post('name'),
								'company'=>$this->input->post('company'),
								'email'=>$this->input->post('email'));
				
				$new_user = $this->model->update_user($id, $save);
				$data['success'] .= ' Your details have been updated.';
			}
		}
		$data['user'] = $this->model->get_user($id);
		$this->view->render('advertisers/change_details.php', $data);
	}
	
	function advertiser_buy_list_slots($page = 1){
		if(!$page)
			$page = 1;
		$this->login_check('advertiser');
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$raw_slots = $this->model->get_paged_slots($offset, $rows);
		if($raw_slots){
			for($i = 0; $i < count($raw_slots); $i++){
				$raw_slots[$i]['prices'] = $this->model->get_slot_prices_order_by_price($raw_slots[$i]['id']);
			}
		}
		$data['banner_slots'] = $raw_slots;
		$data['no_of_slots'] = $this->model->count_slots();
		$data['number_of_pages'] = ceil( $data['no_of_slots'] / $rows );
		$data['page'] = $page;
		
		$this->view->render('advertisers/list_slots_to_buy.php', $data);
	}
	
	function advertiser_buy_slot($id){
		$this->login_check('advertiser');
		$data = array();
		$data['error'] = '';
		$data['success'] = '';
		
		$data['slot'] = $this->model->get_slot($id);
		$data['slot']['is_available'] = $this->is_available($data['slot']);
		
		$data['slot']['place_holder'] = $this->get_banner_place_holder($data['slot']);
		$data['slot']['prices'] = $this->model->get_slot_prices_order_by_price($id);
		
		if(isset($_POST['buy'])){
			if($_POST['title'] == '')
				$data['error'] .= ' The Headline of the banner is required.';
			if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
				$data['error'] .= ' The Destination URL is required.';
			if($_POST['price'] == '')
				$data['error'] .= ' The Price plan is required.';
			if($_POST['quantity'] == '')
				$data['error'] .= ' The Quantity is required.';
			if(APP_USE_PAYPAL == true OR APP_USE_MONEYBOOKERS == true OR APP_USE_BANK_DEPOSIT_TRANSFER == true){
				if($_POST['payment_option'] == '')
					$data['error'] .= ' The Payment option is required.';
			}
			
			if($_POST['advertisement_type'] == 'banner'){
				if($_POST['ad_type'] == 'text' OR (!isset($_POST['ad_type']))){
					if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
						$data['error'] .= ' The Display URL is required.';
					if(($_POST['details1'] == '') AND ($_POST['details2'] == ''))
						$data['error'] .= ' At least one line of description is required.';
					
					if($data['error'] == ''){
						$save['display_url'] = $this->input->post('display_url');
						$save['details1'] = $this->input->post('details1');
						$save['details2'] = $this->input->post('details2');
						$save['advert_type'] = 'text_banner';
					}
				}
			
				if($_POST['ad_type'] == 'image'){
					if($_FILES["image"]["name"]){
						if($_FILES["image"]["error"] > 0){
							$data['error'] .= $_FILES["image"]["error"];
						} else {
							if(!$this->helpers->valid_file_extension($_FILES["image"]["name"]))
								$data['error'] .= ' The uploaded file is of an invalid type (it should be an image or swf).';
							list($image_width, $image_height) = getimagesize($_FILES["image"]["tmp_name"]);
							$selected_slot = $data['slot'];
							if($image_width != $selected_slot['width'])
								$data['error'] .= ' The uploaded file should have a width of ' . $selected_slot['width'] . 'px.';
							if($image_height != $selected_slot['height'])
								$data['error'] .= ' The uploaded file should have a height of ' . $selected_slot['height'] . 'px.';
						}
					} else {
						$data['error'] .= ' You need to upload an image or swf file.';
					}
				
					if($data['error'] == ''){
						$save['advert_type'] = 'image_banner';
						if($_FILES["image"]["name"]){
							$filename = $this->helpers->set_filename("images/adverts/", $_FILES["image"]["name"]);
							$save['image'] = $filename;
						}
					}
				}
			}
			if($_POST['advertisement_type'] == 'textlink'){
				if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
					$data['error'] .= ' The Display URL is required.';
				
				if($data['error'] == ''){
					$save['display_url'] = $this->input->post('display_url');
					$save['advert_type'] = 'textlink';
				}
			}
			
			if($data['error'] == ''){
				$save['title'] = $this->input->post('title');
				$save['destination_url'] = $this->input->post('destination_url');
				$save['price_id'] = $this->input->post('price');
				$save['quantity'] = $this->input->post('quantity');
				$save['advertiser_id'] = $_SESSION['sadvert_user_id'];
				$save['slot_id'] = $id;
				$save['dateadded'] = time();
				if(APP_USE_PAYPAL == true OR APP_USE_MONEYBOOKERS == true OR APP_USE_BANK_DEPOSIT_TRANSFER == true){
					$save['payment_option'] = $this->input->post('payment_option');
				}
				$save['rejected'] = 'n';
				$save['approved'] = 'n';
				$save['confirmed'] = 'n';
				$save['type_of_pending'] = 'purchase';
				
				$pending = $this->model->create_pending($save);
				if($pending){
					if($_FILES["image"]["name"]){
						move_uploaded_file($_FILES["image"]["tmp_name"], "images/adverts/" . $filename);
					}
					header('Location: ' . $this->helpers->create_url('buy_payment', array($this->input->post('payment_option'), $pending, $this->input->post('quantity'), 1)));
					exit;
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		
		
		}
		
		if($data['slot']['advert_type'] == 'textlink'){
			$this->view->render('advertisers/buy_textlink.php', $data);	
		} else {
			$this->view->render('advertisers/buy_banner.php', $data);
		}
	}
	
	function buy_home(){
		$this->buy_list_slots();
	}
	
	function buy_list_slots($page = 1){
		if(!$page)
			$page = 1;
		$rows = 30;
		$offset = ($page - 1) * $rows;
		$data = array();
		$raw_slots = $this->model->get_paged_slots($offset, $rows);
		if($raw_slots){
			for($i = 0; $i < count($raw_slots); $i++){
				$raw_slots[$i]['prices'] = $this->model->get_slot_prices_order_by_price($raw_slots[$i]['id']);
			}
		}
		$data['banner_slots'] = $raw_slots;
		$data['no_of_slots'] = $this->model->count_slots();
		$data['number_of_pages'] = ceil( $data['no_of_slots'] / $rows );
		$data['page'] = $page;
		
		$this->view->render('buy/list_slots_to_buy.php', $data);
	}
	
	function buy_slot($id){
		$data = array();
		$data['error'] = '';
		$data['success'] = '';
		
		$data['slot'] = $this->model->get_slot($id);
		$data['slot']['is_available'] = $this->is_available($data['slot']);
		
		$data['slot']['place_holder'] = $this->get_banner_place_holder($data['slot']);
		$data['slot']['prices'] = $this->model->get_slot_prices_order_by_price($id);
		
		if(isset($_POST['buy'])){
			if($_POST['title'] == '')
				$data['error'] .= ' The Headline of the banner is required.';
			if(($_POST['destination_url'] == '') OR ($_POST['destination_url'] == 'http://'))
				$data['error'] .= ' The Destination URL is required.';
			if($_POST['price'] == '')
				$data['error'] .= ' The Price plan is required.';
			if($_POST['quantity'] == '')
				$data['error'] .= ' The Quantity is required.';
			if(APP_USE_PAYPAL == true OR APP_USE_MONEYBOOKERS == true OR APP_USE_BANK_DEPOSIT_TRANSFER == true){
				if($_POST['payment_option'] == '')
					$data['error'] .= ' The Payment option is required.';
			}
			if($_POST['name'] == '')
				$data['error'] .= ' Your name is required.';
			if($_POST['email'] == '')
				$data['error'] .= ' Your email address is required.';
			if($this->model->email_exists($_POST['email']) == false)
				$data['error'] .= ' The Email is already in use. Please select another one or if you have previously purchased ad space, sign in to the advertisers dashboard and purchase from there.';
			
			if($this->helpers->spamcheck($_POST['email']) == false)
				$data['error'] .= ' The Email Address is invalid.';
			
			if($_POST['advertisement_type'] == 'banner'){
				if($_POST['ad_type'] == 'text' OR (!isset($_POST['ad_type']))){
					if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
						$data['error'] .= ' The Display URL is required.';
					if(($_POST['details1'] == '') AND ($_POST['details2'] == ''))
						$data['error'] .= ' At least one line of description is required.';
					
					if($data['error'] == ''){
						$save['display_url'] = $this->input->post('display_url');
						$save['details1'] = $this->input->post('details1');
						$save['details2'] = $this->input->post('details2');
						$save['advert_type'] = 'text_banner';
					}
				}
			
				if($_POST['ad_type'] == 'image'){
					if($_FILES["image"]["name"]){
						if($_FILES["image"]["error"] > 0){
							$data['error'] .= $_FILES["image"]["error"];
						} else {
							if(!$this->helpers->valid_file_extension($_FILES["image"]["name"]))
								$data['error'] .= ' The uploaded file is of an invalid type (it should be an image or swf).';
							list($image_width, $image_height) = getimagesize($_FILES["image"]["tmp_name"]);
							$selected_slot = $data['slot'];
							if($image_width != $selected_slot['width'])
								$data['error'] .= ' The uploaded file should have a width of ' . $selected_slot['width'] . 'px.';
							if($image_height != $selected_slot['height'])
								$data['error'] .= ' The uploaded file should have a height of ' . $selected_slot['height'] . 'px.';
						}
					} else {
						$data['error'] .= ' You need to upload an image or swf file.';
					}
				
					if($data['error'] == ''){
						$save['advert_type'] = 'image_banner';
						if($_FILES["image"]["name"]){
							$filename = $this->helpers->set_filename("images/adverts/", $_FILES["image"]["name"]);
							$save['image'] = $filename;
						}
					}
				}
			}
			if($_POST['advertisement_type'] == 'textlink'){
				if(($_POST['display_url'] == '') OR ($_POST['display_url'] == 'http://'))
					$data['error'] .= ' The Display URL is required.';
				
				if($data['error'] == ''){
					$save['display_url'] = $this->input->post('display_url');
					$save['advert_type'] = 'textlink';
				}
			}
			
			if($data['error'] == ''){
				$save['name'] = $this->input->post('name');
				$save['email'] = $this->input->post('email');
				$save['company'] = $this->input->post('company');
				$save['title'] = $this->input->post('title');
				$save['destination_url'] = $this->input->post('destination_url');
				$save['price_id'] = $this->input->post('price');
				$save['quantity'] = $this->input->post('quantity');
				$save['slot_id'] = $id;
				$save['dateadded'] = time();
				if(APP_USE_PAYPAL == true OR APP_USE_MONEYBOOKERS == true OR APP_USE_BANK_DEPOSIT_TRANSFER == true){
					$save['payment_option'] = $this->input->post('payment_option');
				}
				
				$save['rejected'] = 'n';
				$save['approved'] = 'n';
				$save['confirmed'] = 'n';
				$save['type_of_pending'] = 'purchase';
				
				$pending = $this->model->create_pending($save);
				if($pending){
					
					
					if($_FILES["image"]["name"]){
						move_uploaded_file($_FILES["image"]["tmp_name"], "images/adverts/" . $filename);
					}
					header('Location: ' . $this->helpers->create_url('buy_payment', array($this->input->post('payment_option'), $pending, $this->input->post('quantity'), 0)));
					exit;
				} else {
					$data['error'] .= ' Something went wrong. Please try again.';
				}
			}
		
		
		}
		
		
		if($data['slot']['advert_type'] == 'textlink'){
			$this->view->render('buy/buy_textlink.php', $data);	
		} else {
			$this->view->render('buy/buy_banner.php', $data);
		}
	}
	
	function buy_contact(){
		$data = array();
		$data['error'] = '';
		if(isset($_POST['send'])){
			if($_POST['subject'] == '')
				$data['error'] .= ' The Subject is required.';
			if($_POST['body'] == '')
				$data['error'] .= ' The Message is required.';
			if($_POST['name'] == '')
				$data['error'] .= ' The Name is required.';
			if($_POST['email'] == '')
				$data['error'] .= ' The Email Address is required.';
			if($this->helpers->spamcheck($_POST['email']) == false)
				$data['error'] .= ' The Email Address is invalid.';
				
			if($data['error'] == ''){
				$email_data = array();
				
				$email_data['name'] = $this->input->post('name');
				$email_data['email'] = $this->input->post('email');
				$email_data['message'] = $this->input->post('body');
				$email_data['subject'] = $this->input->post('subject');
				
				$subject = '[Simple Advert Contact Page]' . strtoupper($this->input->post('subject'));
				$text = $this->view->get('email_templates/contact_text.php', $email_data);
				$html = $this->view->get('email_templates/contact_html.php', $email_data);
				
				$this->helpers->html_email(APP_WEBMASTER_EMAIL, $user['email'], $subject, $text, $html);
				
				$data['success'] .= ' The message has been sent and we will get back to you as soon as possible.';
			}
		}
		$this->view->render('buy/contact.php', $data);
	}
	
	function is_available($slot){
		$return = true;
		if($slot['numberofads'] > 0){
			$no_of_advert = $this->model->count_slot_adverts($slot['id']);
			if($no_of_advert >= $slot['numberofads']){
				$return = false;
			}
		}
		
		return $return;
	}
	
	function get_banner_place_holder($slot){
		$space_height = $slot['height'];
		$padding = ($space_height/2) - 12;
		$height = $space_height - $padding;
		$data = array();
		
		$data['width'] = $slot['width'];
		$data['slot_height'] = $slot['height'];
		$data['height'] = $height;
		$data['padding'] = $padding;
		$data['title'] = '';
		$data['url'] = '#nowhere';
		
		return $this->view->get('advertisements/buy_page_banner_place_holder.php', $data);
	}
	
	function buy_payment($option, $pending, $quantity, $advertiser = 0){
		if(!$advertiser)
			$advertiser = 0;
		$data = array();
		
		$pending_record = $this->model->get_pending($pending);
		$price = $this->model->get_price($pending_record['price_id']);
		
		if($option == 'paypal'){
			
			$paypal=new paypal();
			$paypal->headers_nocache(); 

			if(APP_SANDBOX_PAYMENT === true){
				$paypal->set_to_test();
			}
			
			$paypal->price = ''. $price['price'] . '';
			$paypal->ipn = $this->helpers->create_url('buy_ipn', array('paypal'));
			$paypal->enable_payment();
			$paypal->add('currency_code', APP_CURRENCY_CODE);
			$paypal->add('business', APP_PAYPAL_EMAIL_ADDRESS);
			$paypal->add('item_name','Advertisement Space[' . $_SERVER['SERVER_NAME'] . ']');
			$paypal->add('item_number',''.$pending.'');
			$paypal->add('quantity',intval($quantity));

			$return_url = $this->helpers->create_url('buy_return', array('paypal', $pending, 'success', $advertiser));
			$cancel_url = $this->helpers->create_url('buy_return', array('paypal', $pending, 'cancel', $advertiser));

			$paypal->add('return',$return_url);
			$paypal->add('cancel_return',$cancel_url);
			
			$data['pay_option'] = 'Paypal';
			$data['form'] = $paypal->output_form();
		}
		
		if($option == 'moneybookers'){
			
			$moneybooker=new moneybooker();
			$moneybooker->headers_nocache(); 
			
			if(APP_SANDBOX_PAYMENT === true){
				$moneybooker->set_to_test();
			}

			$total_price = intval($quantity) * $price['price'];
			$moneybooker->price = ''.$total_price.'';
			$moneybooker->ipn = $this->helpers->create_url('buy_ipn', array('moneybookers'));
			$moneybooker->add('currency',APP_CURRENCY_CODE);
			$moneybooker->add('pay_to_email',APP_MONEYBOOKERS_EMAIL_ADDRESS);
			$moneybooker->add('detail1_description',''.intval($quantity).' Advertisement Space[' . $_SERVER['SERVER_NAME'] . ']');
			$moneybooker->add('detail1_text',''.intval($quantity).'Advertisement Space[' . $_SERVER['SERVER_NAME'] . ']');
			$moneybooker->add('transaction_id',''.$pending.'');

			$moneybooker->add('amount',$moneybooker->price);

			$return_url = $this->helpers->create_url('buy_return', array('moneybookers', $pending, 'success', $advertiser));
			$cancel_url = $this->helpers->create_url('buy_return', array('moneybookers', $pending, 'cancel', $advertiser));

			$moneybooker->add('language',APP_MONEYBOOKERS_LANGUAGE);
			$moneybooker->add('status_url',$moneybooker->ipn);
			$moneybooker->add('return_url',$return_url);
			$moneybooker->add('cancel_url',$cancel_url);

			$data['pay_option'] = 'Moneybookers';
			$data['form'] = $moneybooker->output_form();
		}
		
		if($option == 'bank_deposit'){
			$data['amount_to_pay'] = intval($quantity) * $price['price'];
			$data['heading'] = "Purchase by Bank Deposit/Transfer";
			$this->view->render('buy/bank_deposit_complete.php', $data);
		} else {
			$this->view->render('buy/payment.php', $data);
		}
	}
	
	function buy_return($option, $pending, $state , $advertiser = 0){
		if(!$advertiser)
			$advertiser = 0;
		$data = array();
		if($state == 'success'){
			$data['message'] = 'The transaction was successfull. You advertisement will be up and running as soon as we approve it. Thank you.';
			$data['heading'] = 'Thank you!';
		} else {
			$data['message'] = 'The transaction has been cancelled.';
			$data['heading'] = 'Transaction cancelled.';
			$pending_advert = $this->model->get_pending($pending);
			if($pending_advert['image']){
				$file = "images/adverts/" . $pending_advert['image'];
				$exists = file_exists($file);
				if($exists){
						unlink($file);
				}
			}
			$this->model->delete_pending($pending);
		}
		if($advertiser == 1){
			$this->view->render('advertisers/buy_return.php', $data);
		} else {
			$this->view->render('buy/return.php', $data);
		}
	}
	
	function buy_ipn($option){
		$completed = false;
		if($option == 'paypal'){
			$paypal=new paypal();
			$paypal->log=1;
			if($paypal->validate_ipn() AND $paypal->payment_success==1){
				$id = intval($paypal->posted_data['item_number']);
				$this->model->update_pending($id, array('confirmed'=>'y'));
				$completed = true;
			}
		}
		
		if($option == 'moneybookers'){
			$moneybooker=new moneybooker();
			$moneybooker->log=1;
			if($moneybooker->validate_ipn() AND (intval($moneybooker->posted_data['status'])==2)){
				$id=intval($moneybooker->posted_data['transaction_id']);
				$this->model->update_pending($id, array('confirmed'=>'y'));
				$completed = true;
			}
		}
		
		if($completed == true){
			if(APP_AUTO_APPROVE_ON_IPN == true){
				$this->approve_pending($id);
				$subject = '[Simple Advert Script]PENDING APPROVAL';
				$message = 'There is an advertisement auto approved at '. $_SERVER['SERVER_NAME'];
				$from = APP_WEBMASTER_EMAIL;
				$headers = "From:" . $from;
				$to = APP_WEBMASTER_EMAIL;
				mail($to, $subject, $message, $headers);
			} else {
				$subject = '[Simple Advert Script]PENDING APPROVAL';
				$message = 'There is an advertisement pending approval at '. $_SERVER['SERVER_NAME'];
				$from = APP_WEBMASTER_EMAIL;
				$headers = "From:" . $from;
				$to = APP_WEBMASTER_EMAIL;
				mail($to, $subject, $message, $headers);
			}
		}
		exit();
	}
	
	function ajax(){
		$output = array();
		$output['status'] = 0;
		$id = 0;
		$id_s = array();
		
		foreach($_GET as $key=>$value){
			if($key != 'simple_ad_count'){
				$space_array = explode('_', $value);
				$id = $space_array[1];
				if(($space_array[0] == 'sa') AND (is_numeric($space_array[1]))){
					$id_s[$space_array[1]][] = $value;
					
				}
			}
		}
		
		foreach($id_s as $k=>$v){
			if(count($v) == 1){
				$output['adverts'][$v[0]] = $this->get_markup($this->helpers->clean($k));
			} else {
				$many_markup = $this->get_markup($this->helpers->clean($k), count($v));
				$v_count = 0;
				while($v_count < count($many_markup)){
					$output['adverts'][$v[$v_count]] = $many_markup[$v_count];
					$v_count++;
				}
			}
		}
		
		$output['status'] = 1;
		
		header("Content-type: text/plain");
		echo $_GET['callback'] . '(' . json_encode($output). ')';
		exit();
	}
	
	function get_markup($id, $how_many = 1){
		if(!$how_many)
			$how_many = 1;
		$result = 'none';
		$advert_id = 0;
		$impressions = 0;
		
		$space = $this->model->get_slot($id);
		if($space AND $space['advert_type'] == 'banner'){
			$temp_adverts = $this->model->get_slot_runnning_adverts($id);
			$adverts = array();
			for($i = 0; $i < count($temp_adverts); $i++){
				if($this->is_live($temp_adverts[$i]))
					$adverts[] = $temp_adverts[$i];
			}
			
			if(count($adverts) > 0){
				if($space['banner_type'] == 'multi_text'){
					if(count($adverts) > 1){
						$number_of_ads = floor($space['height'] / 90);
						$adverts_index = $this->randomize_adverts($adverts, $number_of_ads);
						
						$data = array();
						$data['width'] = $space['width'];
						$data['height'] = $space['height'];
						$data['ads'] = array();
						
						foreach($adverts_index as $index){
							$data['ads'][] = array('id'=>$adverts[$index]['id'],
													'title'=>$adverts[$index]['title'],
													'display_url'=>$adverts[$index]['display_url'],
													'details1'=>$adverts[$index]['details1'],
													'details2'=>$adverts[$index]['details2']);
							$this->model->update_impressions($adverts[$index]);
						}
						$result = $this->view->get('advertisements/multi_text_advert_markup.php', $data);
						
					} elseif(count($adverts) == 1) {
						$index = 0;
						$the_top_margin = 0;
						$data = array();
						$data['width'] = $space['width'];
						$data['height'] = $space['height'];
					
					
						$data['id'] = $adverts[$index]['id'];
						$data['title'] = $adverts[$index]['title'];
						$data['display_url'] = $adverts[$index]['display_url'];
						$data['details1'] = '';
						$data['details2'] = '';
					
						if($adverts[$index]['details1']){
							$data['details1'] = $adverts[$index]['details1'];
							$the_top_margin = $the_top_margin + 20;
						}
						if($adverts[$index]['details1']){
							$data['details2'] = $adverts[$index]['details2'];
							$the_top_margin = $the_top_margin + 20;
						}
					
						if($space['height'] >= 40){
							$data['margin_top'] = ($space['height'] / 2) - $the_top_margin;
						} else {
							$data['margin_top'] = 0;
						}
					
						$result = $this->view->get('advertisements/text_advert_markup.php', $data);
						$this->model->update_impressions($adverts[$index]);
					}
				} else {
					if($how_many == 1){
						if(count($adverts) > 1){
						$index = $this->randomize_adverts($adverts);
						} elseif(count($adverts) == 1) {
							$index = 0;
						}
			
						$impressions = $adverts[$index]['impressions'];
				
						if($adverts[$index]['image']){
							$data = array();
							$data['width'] = $space['width'];
							$data['height'] = $space['height'];
							if($this->helpers->get_extension($adverts[$index]['image']) == 'swf'){
								$data['swf'] = $adverts[$index]['image'];
								$result = $this->view->get('advertisements/swf_advert_markup.php', $data);
							} else {
								$data['id'] = $adverts[$index]['id'];
								$data['title'] = $adverts[$index]['title'];
								$data['image'] = $adverts[$index]['image'];
								$result = $this->view->get('advertisements/image_advert_markup.php', $data);
							}
					
						} else {
							$the_top_margin = 0;
							$data = array();
							$data['width'] = $space['width'];
							$data['height'] = $space['height'];
					
					
							$data['id'] = $adverts[$index]['id'];
							$data['title'] = $adverts[$index]['title'];
							$data['display_url'] = $adverts[$index]['display_url'];
							$data['details1'] = '';
							$data['details2'] = '';
					
							if($adverts[$index]['details1']){
								$data['details1'] = $adverts[$index]['details1'];
								$the_top_margin = $the_top_margin + 20;
							}
							if($adverts[$index]['details1']){
								$data['details2'] = $adverts[$index]['details2'];
								$the_top_margin = $the_top_margin + 20;
							}
					
							if($space['height'] >= 40){
								$data['margin_top'] = ($space['height'] / 2) - $the_top_margin;
							} else {
								$data['margin_top'] = 0;
							}
					
							$result = $this->view->get('advertisements/text_advert_markup.php', $data);
						}
						$this->model->update_impressions($adverts[$index]);
					} else {
						$result = array();
						$adverts_index = $this->randomize_adverts($adverts, $how_many);
						foreach($adverts_index as $index){
							if($adverts[$index]['image']){
								$data = array();
								$data['width'] = $space['width'];
								$data['height'] = $space['height'];
								if($this->helpers->get_extension($adverts[$index]['image']) == 'swf'){
									$data['swf'] = $adverts[$index]['image'];
									$result = $this->view->get('advertisements/swf_advert_markup.php', $data);
								} else {
									$data['id'] = $adverts[$index]['id'];
									$data['title'] = $adverts[$index]['title'];
									$data['image'] = $adverts[$index]['image'];
									$result[] = $this->view->get('advertisements/image_advert_markup.php', $data);
								}
					
							} else {
								$the_top_margin = 0;
								$data = array();
								$data['width'] = $space['width'];
								$data['height'] = $space['height'];
					
					
								$data['id'] = $adverts[$index]['id'];
								$data['title'] = $adverts[$index]['title'];
								$data['display_url'] = $adverts[$index]['display_url'];
								$data['details1'] = '';
								$data['details2'] = '';
					
								if($adverts[$index]['details1']){
									$data['details1'] = $adverts[$index]['details1'];
									$the_top_margin = $the_top_margin + 20;
								}
								if($adverts[$index]['details1']){
									$data['details2'] = $adverts[$index]['details2'];
									$the_top_margin = $the_top_margin + 20;
								}
					
								if($space['height'] >= 40){
									$data['margin_top'] = ($space['height'] / 2) - $the_top_margin;
								} else {
									$data['margin_top'] = 0;
								}
					
								$result[] = $this->view->get('advertisements/text_advert_markup.php', $data);
							}	
							$this->model->update_impressions($adverts[$index]);
						}
					}
					
					
				}
			}
		}
		return $result;
	}
	
	function randomize_adverts($array, $number = 1){
		if(!$number)
			$number = 1;
		$i = 0;
		$count = count($array);
		if($number == 1){
			$index_for_oldest = 0;
			$impressions_total = 0;
			
			if($count == 1)
				return 0;
			
			$max_rand = $count - 1;
			return rand(0,$max_rand);
			
			/*
			while($i < $count){
				$total_latest_impressions = $this->model->get_latest_impression($array[$i]['id']);
				if(($impressions_total == 0) AND ($i == 0)){
					$impressions_total = $total_latest_impressions;
					$index_for_oldest = $i;
				} else {
					if($total_latest_impressions < $impressions_total){
						$impressions_total = $total_latest_impressions;
						$index_for_oldest = $i;
					}
				}
				$i++;
			}
		
			return $index_for_oldest; */
		} else {
			$keys = array();
			if($count > $number){
				$max = $count - 1;
				$selected = array();
				while($i < $number){
					$random = rand(0, $max);
					if(!in_array($random, $keys)) {
						$keys[] = $random;
						$i++;
					}
				}
				return $keys;
			} else {
				while($i < $count){
					$keys[] = $i;
					$i++;
				}
				return $keys;
			}
		}
	}
	
	function statistics($id, $redirect = 0){
		if(!$redirect)
			$redirect = 0;
		$id = intval(str_replace("simple_textlink_","",$id));
		$advert = $this->model->get_advert($id);
		$this->model->update_clicks($advert);
		if($redirect == 1){
			header('Location: '.$advert['destination_url']);
			exit();
		}
		return true;
	}
	
	/*
	*	This method check if a user is logged in as an admin. It restrics access to the methods that handle the admin dashboard
	*/
	function login_check($role = 'admin'){	
		if(!$_SESSION['sadvert_logged_in']){
			header('Location: ' . $this->helpers->create_url('login'));
			exit;
		} else {
			if($_SESSION['sadvert_user_role'] != $role){
				header('Location: ' . $this->helpers->create_url('forbidden_page'));
				exit;
			} else {
				return true;
			}
		}
	}
	
	/*
	*	THE FOLLOWING METHODS ARE FOR USER AUTHENTICATION FOR THE ADMIN/ADVERTISERS' DASHBOARD
	*/
	
	
	function logout(){
		unset($_SESSION['sadvert_logged_in']);
		unset($_SESSION['sadvert_user_id']);
		unset($_SESSION['sadvert_user_role']);
		header('Location: /');
		exit;
	}
	
	function login(){
		$data = array();
		$data['error'] = '';
		if(isset($_POST['login'])){
			$username = $this->input->post('username');
			$password = md5($this->input->post('password'));
			$user = $this->model->check_login($username, $password);
			if($user){
				$_SESSION['sadvert_logged_in'] = true;
				$_SESSION['sadvert_user_id'] = $user['id'];
				$_SESSION['sadvert_user_role'] = $user['role'];
				if($user['role'] == 'admin'){
					
					header('Location: ' . $this->helpers->create_url('admin_home'));
					exit;
				} else {
					header('Location: ' . $this->helpers->create_url('advertiser_home'));
					exit;
				}
			} else {
				$data['error'] = ' The login information is not correct.';
			}
		}
		$this->view->render('auth/login.php', $data);
	}
	
	function changepassword(){
		$this->login_check($_SESSION['sadvert_user_role']);
		$data = array();
		$data['error'] = '';
		if(isset($_POST['changepass'])){
			if($_POST['o_password'] == '')
				$data['error'] .= ' The old password is required.';
			if($_POST['n_password'] == '')
				$data['error'] .= ' The new password is required.';
			if($_POST['c_password'] != $_POST['n_password'])
				$data['error'] .= ' Check the new password. It was not confirmed correctly.';
			if(!$this->model->check_password($_SESSION['sadvert_user_id'], md5($this->input->post('o_password'))))
				$data['error'] .= ' The old password you entered is not correct.';
			
			if($data['error'] == ''){
				$password = md5($this->input->post('n_password'));
				$this->model->change_password($_SESSION['sadvert_user_id'], $password);
				
				if($_SESSION['sadvert_user_role'] == 'admin'){
					$data['success'] = 'The password has been changed. <a href="' . $this->helpers->create_url('admin_home') . '">Click here</a> to go back to the home page.';
				} else {
					$data['success'] = 'The password has been changed. <a href="' . $this->helpers->create_url('advertiser_home') . '">Click here</a> to go back to the home page.';
				}
				
			}
		}
		$this->view->render('auth/change.php', $data);
	}
	
	function forgotpassword(){
		$data = array();
		if(isset($_POST['forgot'])){
			$username = $this->input->post('username');
			$user = $this->model->get_user_using_email($this->input->post('username'));
			if($user){
				$password = $this->generatepassword();
				
				$to = $user['email'];
				$subject = "Simple Advert: User Authentication";
				$message = "Your new password is: " . $password;
				$from = APP_WEBMASTER_EMAIL;
				$headers = "From: $from";
		
				$this->model->change_password($user['id'], md5($password));
		
				if(mail($to,$subject,$message,$headers)){
					$data['success'] = 'The password has been reset. Check your email for the new one.';
				} else {
					$data['error'] = ' Something went wrong. Please try again.';
				}
			} else{
				$data['error'] = ' The email you entered is not correct.';
			}
		}
		$this->view->render('auth/forgot.php', $data);
	}
	
	function generatepassword($len = 8){
		$pool = '123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$str = '';
		for ($i = 0; $i < $len; $i++)
		{
			$str .= substr($pool, mt_rand(0, strlen($pool) -1), 1);
		}
		return $str;
	}
	
	function forbidden_page(){
		$this->view->render('forbidden.php');
	}
	
	function error_404(){
		$data = array();
		$this->view->render('error_404.php', $data);
	}
}
